package com.sun.jmx.mbeanserver;

import javax.management.InstanceAlreadyExistsException;
import javax.management.MBeanServer;
import java.util.Iterator;
import java.util.Set;
import javax.management.JMX;
import javax.management.NotCompliantMBeanException;
import javax.management.ObjectName;

public class MXBeanSupport extends MBeanSupport<ConvertingMethod>
{
    private final Object lock;
    private MXBeanLookup mxbeanLookup;
    private ObjectName objectName;
    
    public <T> MXBeanSupport(T t, Class<T> clazz) throws NotCompliantMBeanException {
        super(t, clazz);
        this.lock = new Object();
    }
    
    @Override
    MBeanIntrospector<ConvertingMethod> getMBeanIntrospector() {
        return MXBeanIntrospector.getInstance();
    }
    
    @Override
    Object getCookie() {
        return this.mxbeanLookup;
    }
    
    static <T> Class<? super T> findMXBeanInterface(Class<T> clazz) {
        if (clazz == null) {
            throw new IllegalArgumentException("Null resource class");
        }
        Set<Class<?>> transitiveInterfaces = transitiveInterfaces(clazz);
        Set<Class> set = Util.newSet();
        for (Class interfaceClass : transitiveInterfaces) {
            if (JMX.isMXBeanInterface(interfaceClass)) {
                set.add(interfaceClass);
            }
            else {
                continue;
            }
        }
    Label_0070:
        while (set.size() > 1) {
            for (Class clazz2 : set) {
                Iterator<Object> iterator3 = set.iterator();
                while (iterator3.hasNext()) {
                    Class<Object> clazz3 = (Class)iterator3.next();
                    if (clazz2 != clazz3 && clazz3.isAssignableFrom(clazz2)) {
                        iterator3.remove();
                        continue Label_0070;
                    }
                    continue;
                }
            }
            throw new IllegalArgumentException("Class " + clazz.getName() + " implements more than one MXBean interface: " + set);
        }
        if (set.iterator().hasNext()) {
            return (Class)Util.cast(set.iterator().next());
        }
        throw new IllegalArgumentException("Class " + clazz.getName() + " is not a JMX compliant MXBean");
    }
    
    private static Set<Class<?>> transitiveInterfaces(Class<?> clazz) {
        Set<Class<?>> set = Util.newSet();
        transitiveInterfaces(clazz, set);
        return set;
    }
    
    private static void transitiveInterfaces(Class<?> clazz, Set<Class<?>> set) {
        if (clazz == null) {
            return;
        }
        if (clazz.isInterface()) {
            set.add(clazz);
        }
        transitiveInterfaces(clazz.getSuperclass(), set);
        Class[] interfaces = clazz.getInterfaces();
        for (int length = interfaces.length, i = 0; i < length; ++i) {
            transitiveInterfaces(interfaces[i], set);
        }
        return;
    }
    
    @Override
    public void register(MBeanServer mBeanServer, ObjectName objectName) throws InstanceAlreadyExistsException {
        if (objectName == null) {
            throw new IllegalArgumentException("Null object name");
        }
        synchronized (this.lock) {
            (this.mxbeanLookup = MXBeanLookup.lookupFor(mBeanServer)).addReference(objectName, this.getResource());
            this.objectName = objectName;
        }
        return;
    }
    
    @Override
    public void unregister() {
        synchronized (this.lock) {
            if (this.mxbeanLookup != null && this.mxbeanLookup.removeReference(this.objectName, this.getResource())) {
                this.objectName = null;
            }
        }
    }
}
